import logging
import json
import grpc
from zeebe_grpc import gateway_pb2, gateway_pb2_grpc


class ZeebeClient:
    """Represents a zeebe Client"""

    def __init__(self, brocker):
        self.brocker = brocker

    def get_topology(self):
        with grpc.insecure_channel(self.brocker) as channel:
            stub = gateway_pb2_grpc.GatewayStub(channel)

            return stub.Topology(gateway_pb2.TopologyRequest())

    def deploy_workflow(self, name, workflow):
        workflow = gateway_pb2.WorkflowRequestObject(
            name=name,
            type=gateway_pb2.WorkflowRequestObject.BPMN,
            definition=workflow
        )

        with grpc.insecure_channel(self.brocker) as channel:
            stub = gateway_pb2_grpc.GatewayStub(channel)

            return stub.DeployWorkflow(gateway_pb2.DeployWorkflowRequest(workflows=[workflow]))

    def create_workflow_instance(self, process_id, variables, version=-1):
        with grpc.insecure_channel(self.brocker) as channel:
            stub = gateway_pb2_grpc.GatewayStub(channel)

            return stub.CreateWorkflowInstance(
                 gateway_pb2.CreateWorkflowInstanceRequest(
                     bpmnProcessId=process_id,
                     version=version,
                     variables=json.dumps(variables)
                 )
            )

    def create_workflow_instance_with_result(self, process_id, variables, version=-1):
        with grpc.insecure_channel(self.brocker) as channel:
            stub = gateway_pb2_grpc.GatewayStub(channel)

            return stub.CreateWorkflowInstanceWithResult(
                gateway_pb2.CreateWorkflowInstanceRequest(
                    bpmnProcessId=process_id,
                    version=version,
                    variables=json.dumps(variables)
                )
            )
